@@ -1,23 +1,51 @@
 package HTML::TableContent::Element;
 
 use Moo;
+use HTML::TableContent::Table;
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
+
+my @ATTRIBUTE = qw/class id style colspan rowspan/;
 
 around BUILDARGS => sub {
     my ( $orig, $class, $args ) = @_;
+    
+    my $build = ( );
 
-    return $class->$orig( attributes => $args );
+    $build->{attributes} = $args;
+    $build->{attribute_list} = \@ATTRIBUTE;
+
+    for my $field ( @ATTRIBUTE ) {
+        if (defined $args->{$field}) {
+            $build->{$field} = $args->{$field};
+        }
+    }
+    
+    return $class->$orig( $build );
 };
 
+for my $field (@ATTRIBUTE) {
+    has $field => (
+        is => 'rw',
+        lazy => 1,
+        trigger => 1,
+        default => q{}
+    );
+}
+
 has attributes => (
     is      => 'rw',
     default => sub { {} }
 );
 
+has attribute_list => (
+    is      => 'rw',
+);
+
 has data => (
     is      => 'rw',
-    default => sub { [] }
+    clearer => 1,
+    builder => 1,
 );
 
 has nested => (
@@ -25,6 +53,11 @@ has nested => (
     default => sub { [] },
 );
 
+has html_tag => (
+    is => 'rw',
+    default => sub { return 'table'; },
+);
+
 sub has_nested { return scalar @{ $_[0]->nested } ? 1 : 0; }
 
 sub count_nested { return scalar @{ $_[0]->nested }; }
@@ -35,23 +68,89 @@ sub get_nested { return $_[0]->nested->[ $_[1] ]; }
 
 sub all_nested { return @{ $_[0]->nested }; }
 
+sub add_nested { 
+    my $table = HTML::TableContent::Table->new($_[1]);
+    push @{ $_[0]->nested }, $table;
+    return $table;
+}
+
+sub add_to_nested {
+    return push @{ $_[0]->nested}, $_[1];
+}
+
 sub text { return join q{ }, @{ $_[0]->data }; }
 
-sub lc_text { my $text = join q{ }, @{ $_[0]->data }; return lc $text; }
+sub add_text { return push @{ $_[0]->data }, $_[1]; }
 
-sub class { return $_[0]->attributes->{class}; }
+sub lc_text { return lc $_[0]->text; }
 
-sub id { return $_[0]->attributes->{id}; }
+sub add_class { my $class = $_[0]->class; return $_[0]->class(sprintf('%s %s', $class, $_[1])); }
+
+sub add_style { my $style = $_[0]->style; return $_[0]->style(sprintf('%s %s', $style, $_[1])); }
 
 sub raw {
     my $args = $_[0]->attributes;
+    
+    if ($_[0]->has_nested) {
+        $args->{nested} = ( );
+        for ( $_[0]->all_nested ) {
+            push @{ $args->{nested} }, $_->raw;
+        }
+    }
+    
     if ( scalar @{ $_[0]->data } ) {
         $args->{text} = $_[0]->text;
         $args->{data} = $_[0]->data;
     }
+    
     return $args;
 }
 
+sub render {
+    my $args = $_[0]->attributes;
+    
+    my $attr = '';
+    foreach my $attribute (@{ $_[0]->attribute_list }) {
+        if (my $val = $args->{$attribute}) {
+            $attr .= sprintf '%s="%s" ', $attribute, $val;
+        }
+    }
+
+    my $tag = $_[0]->html_tag;
+    return $_[0]->tidy_html(sprintf("<%s %s>%s</%s>", $tag, $attr, $_[0]->_render_element, $tag));
+}
+
+sub _render_element {
+    return $_[0]->text;
+}
+
+sub _trigger_class { return $_[0]->attributes->{class} = $_[1]; }
+
+sub _trigger_id { return $_[0]->attributes->{id} = $_[1]; }
+
+sub _trigger_style { return $_[0]->attributes->{style} = $_[1]; }
+
+sub _trigger_colspan { return $_[0]->attributes->{colspan} = $_[1]; }
+
+sub _trigger_rowspan { return $_[0]->attributes->{rowspan} = $_[1]; }
+
+sub _build_data {
+    my $data = delete $_[0]->attributes->{data};
+    my $text = delete $_[0]->attributes->{text};
+
+    if ( defined $text ) {
+        push @{ $data }, $text;
+    }
+
+    return $data if defined $data && scalar @{ $data };
+    return [ ];
+}
+
+sub tidy_html {
+    $_[1] =~ s/\s+>/>/g;
+    return $_[1];
+}
+
 __PACKAGE__->meta->make_immutable;
 
 1;
@@ -64,7 +163,7 @@ HTML::TableContent::Element - attributes, text, data, class, id
 
 =head1 VERSION
 
-Version 0.07
+Version 0.08
 
 =cut
 
@@ -117,6 +216,48 @@ Element tag's id if found.
 
     $element->id;
 
+=head2 nested
+
+ArrayRef of nested Tables.
+
+    $element->nested
+
+=head2 all_nested
+
+Array of nested Tables.
+
+    $element->all_nested
+
+=head2 add_nested
+
+Add a nested L<HTML::TableContent::Table> to the element.
+
+    $element->add_nested({});
+
+=head2 has_nested
+
+Boolean check, returns true if the element has nested tables.
+
+    $element->has_nested
+
+=head2 count_nested
+
+Count number of nested tables.
+
+    $element->count_nested
+
+=head2 get_first_nested
+
+Get the first nested table.
+
+    $element->get_first_nested
+
+=head2 get_nested
+
+Get Nested table by index.
+
+    $element->get_nested(1);
+
 =head1 AUTHOR
 
 LNATION, C<< <thisusedtobeanemail at gmail.com> >>
