@@ -2,24 +2,20 @@ package HTML::TableContent::Parser;
 
 use Moo;
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 
 extends 'HTML::Parser';
 
 use HTML::TableContent::Table;
-use HTML::TableContent::Table::Header;
-use HTML::TableContent::Table::Row;
-use HTML::TableContent::Table::Row::Cell;
-use HTML::TableContent::Table::Caption;
 
-has [qw(current_tables nested)] => (
+has [qw(current_tables nested caption_selectors)] => (
     is      => 'rw',
     lazy    => 1,
     clearer => 1,
     default => sub { [] },
 );
 
-has [qw(current_table current_element)] => (
+has [qw(current_table current_element selected)] => (
     is      => 'rw',
     lazy    => 1,
     clearer => 1,
@@ -31,6 +27,8 @@ has options => (
     builder => 1,
 );
 
+sub has_caption_selector { return scalar @{ $_[0]->caption_selectors } ? 1 : 0 }
+
 sub count_nested { return scalar @{ $_[0]->nested }; }
 
 sub has_nested { return $_[0]->count_nested ? 1 : 0; }
@@ -49,28 +47,6 @@ sub current_or_nested {
     return $_[0]->has_nested ? $_[0]->get_last_nested : $_[0]->current_table;
 }
 
-sub current_cell_header {
-    my ( $self, $current_cell ) = @_;
-
-    my $table = $self->current_or_nested;
-    my $row   = $table->get_last_row;
-
-    my $header;
-    if ( $row->header ) {
-        $header = $row->header;
-    }
-    else {
-        my $cell_index = $table->get_last_row->cell_count;
-        $header = $table->headers->[$cell_index];
-    }
-
-    return unless $header;
-
-    push @{ $header->cells }, $current_cell;
-
-    return $header;
-}
-
 sub parse {
     my ( $self, $data ) = @_;
 
@@ -91,47 +67,28 @@ sub start {
     my ( $self, $tag, $attr, $attrseq, $origtext ) = @_;
 
     $tag = lc $tag;
-
-    if ( my $store_tag = $self->options->{$tag} ) {
-        my $class   = $store_tag->{class};
-        my $element = $class->new($attr);
-
-        $self->current_element($element);
-
+    if ( my $option = $self->options->{$tag} ) {
         my $table = $self->current_or_nested;
+        my $action = $option->{add};
+        my $element = $self->$action($attr, $table);
+        $self->current_element($element);
+    }
 
-        if ( $tag eq 'th' ) {
-            $table->get_last_row->header($element);
-            push @{ $table->headers }, $element;
-        }
-        if ( $tag eq 'tr' ) {
-            push @{ $table->rows }, $element;
-        }
-        if ( $tag eq 'td' ) {
-            $element->header( $self->current_cell_header($element) );
-            push @{ $table->get_last_row->cells }, $element;
-        }
-
-        if ( $tag eq 'caption' ) {
-            $table->caption($element);
-        }
-
-        if ( $tag eq 'table' ) {
-            if ( defined $self->current_table
-                && $self->current_table->isa('HTML::TableContent::Table') )
-            {
-                if ( $self->has_nested ) {
-
-              # first table has all nested tables - $tp->get_first_table->nested
-                    push @{ $self->current_table->nested }, $element;
-                }
-                push @{ $self->nested },  $element;
-                push @{ $table->nested }, $element;
-                push @{ $table->get_last_row->get_last_cell->nested }, $element;
-            }
-            else {
-                $self->current_table($element);
+    if ( $self->has_caption_selector ) {
+        foreach my $selector ( @{ $self->caption_selectors }) {
+            if ( $tag eq $selector ) {
+                $self->selected({});
             }
+            
+            for my $field (qw/id class/) {
+               my $val = $attr->{$field};
+               
+               next unless $val;
+               
+               if ( $val =~ /$selector/) {
+                    $self->selected($attr);
+               }
+           }
         }
     }
 
@@ -147,6 +104,15 @@ sub text {
             push @{ $elem->data }, $text;
         }
     }
+    if ( my $selected = $self->selected) {
+        
+        return if $selected->{text};
+
+        if ( $text =~ m{\S+}xms ) {
+            $selected->{text} = $text;
+            $self->selected($selected);
+        }
+    }
 
     return;
 }
@@ -156,66 +122,130 @@ sub end {
 
     $tag = lc $tag;
 
-    if ( $tag eq 'table' ) {
-        if ( $self->has_nested ) {
-            $self->clear_last_nested;
-        }
-        else {
-            push @{ $self->current_tables }, $self->current_table;
-            $self->clear_current_table;
+    if ( my $option = $self->options->{$tag} ) {
+        my $table = $self->current_or_nested;
+        if ( my $action = $option->{close} ) {
+            my $element = $self->$action($table);
         }
     }
 
-    if ( $tag eq 'tr' ) {
-        my $table =
-          $self->has_nested ? $self->get_last_nested : $self->current_table;
-
-        my $row = $table->get_last_row;
-
-        if ( $row->cell_count == 0 ) {
-            $table->clear_last_row;
-        }
-
-        if ( $row->header ) {
-            $table->clear_last_row;
-
-            my $index = 0;
-            foreach my $cell ( $row->all_cells ) {
-                my $row = $table->rows->[$index];
-                if ( defined $row ) {
-                    push @{ $row->cells }, $cell;
-                }
-                else {
-                    my $new_row = HTML::TableContent::Table::Row->new();
-                    push @{ $new_row->cells }, $cell;
-                    push @{ $table->rows },    $new_row;
-                }
-                $index++;
-            }
-        }
-    }
+    return;
 }
 
 sub _build_options {
     return {
         table => {
-            class => 'HTML::TableContent::Table',
+            add => '_add_table',
+            close => '_close_table',
         },
         th => {
-            class => 'HTML::TableContent::Table::Header',
+            add => '_add_header',
         },
         tr => {
-            class => 'HTML::TableContent::Table::Row',
+            add => '_add_row',
+            close => '_close_row',
         },
         td => {
-            class => 'HTML::TableContent::Table::Row::Cell',
+            add => '_add_cell',
         },
         caption => {
-            class => 'HTML::TableContent::Table::Caption',
+            add => '_add_caption'
         }
     };
 }
 
+sub _add_header {
+    my ($self, $attr, $table) = @_;
+
+    my $header = $table->add_header($attr);
+    $table->get_last_row->header($header);
+    return $header;
+}
+
+sub _add_row {
+    my ($self, $attr, $table) = @_;
+
+    my $row = $table->add_row($attr);
+    return $row;
+}
+
+sub _add_cell {
+    my ($self, $attr, $table) = @_;
+
+    my $cell = $table->get_last_row->add_cell($attr);
+    $table->parse_to_column($cell);
+    return $cell;
+}
+
+sub _add_caption {
+    my ($self, $attr, $table) = @_;
+
+    my $caption = $table->add_caption($attr);
+    return $caption;
+}
+
+sub _add_table {
+    my ($self, $attr, $table) = @_;
+
+    my $element = HTML::TableContent::Table->new($attr);
+
+    if ( defined $table && $table->isa('HTML::TableContent::Table') ) {
+        if ( $self->has_nested ) {
+            push @{ $self->current_table->nested }, $element; 
+        }
+        push @{ $self->nested }, $element;
+        push @{ $table->nested }, $element;
+        push @{ $table->get_last_row->get_last_cell->nested }, $element;
+    }
+    else {
+        if ( my $caption = $self->selected ){
+            $element->add_caption($caption);
+            $self->clear_selected;
+        }
+        $self->current_table($element);
+    }
+}
+
+sub _close_table {
+    my ($self, $table) = @_;
+
+    if ( $self->has_nested ) {
+        return $self->clear_last_nested;
+    }
+    else {
+        push @{ $self->current_tables }, $self->current_table;
+        return $self->clear_current_table;
+    }
+}
+
+sub _close_row {
+    my ($self, $table) = @_;
+
+    my $row = $table->get_last_row;
+
+    if ( $row->header ) {
+        $table->clear_last_row;
+
+        my $index = 0;
+        foreach my $cell ( $row->all_cells ) {
+            my $row = $table->rows->[$index];
+            if ( defined $row ) {
+                push @{ $row->cells }, $cell;
+            }
+            else {
+                my $new_row = $table->add_row({});
+                push @{ $new_row->cells }, $cell;
+            }
+            $index++;
+        }
+    }
+    elsif ( $row->cell_count == 0 ) {
+        $table->clear_last_row;
+    }
+
+    return;
+}
+
 1;
 
 __END__
@@ -226,7 +256,7 @@ HTML::TableContent::Parser - HTML::Parser subclass.
 
 =head1 VERSION
 
-Version 0.07
+Version 0.08
 
 =cut
 
