@@ -2,11 +2,15 @@ package HTML::TableContent::Table;
 
 use Moo;
 
-our $VERSION = '0.07';
+use HTML::TableContent::Table::Caption;
+use HTML::TableContent::Table::Header;
+use HTML::TableContent::Table::Row;
+
+our $VERSION = '0.08';
 
 extends 'HTML::TableContent::Element';
 
-has caption => ( is => 'rw', );
+has caption => ( is => 'rw', lazy => 1 );
 
 has [qw(headers rows)] => (
     is      => 'rw',
@@ -14,8 +18,26 @@ has [qw(headers rows)] => (
     default => sub { [] },
 );
 
+has '+html_tag' => (
+    default => 'table',
+);
+
+sub add_caption { 
+    my $caption = HTML::TableContent::Table::Caption->new($_[1]);   
+    $_[0]->caption($caption);
+    return $caption;
+}
+
+sub has_caption { return $_[0]->caption ? 1 : 0 };
+
 sub all_rows { return @{ $_[0]->rows }; }
 
+sub add_row { 
+    my $row = HTML::TableContent::Table::Row->new($_[1]);
+    push @{ $_[0]->rows }, $row;
+    return $row;
+}
+
 sub row_count { return scalar @{ $_[0]->rows }; }
 
 sub get_row { return $_[0]->rows->[ $_[1] ]; }
@@ -24,31 +46,33 @@ sub get_first_row { return $_[0]->get_row(0); }
 
 sub get_last_row { return $_[0]->get_row( $_[0]->row_count - 1 ); }
 
-sub clear_last_row { return delete $_[0]->rows->[ $_[0]->row_count - 1 ]; }
+sub clear_row { return splice @{ $_[0]->rows }, $_[1], 1;  }
+
+sub clear_first_row { return shift @{ $_[0]->rows }; }
+
+sub clear_last_row { return $_[0]->clear_row($_[0]->row_count - 1); }
 
 sub all_headers { return @{ $_[0]->headers }; }
 
+sub add_header { 
+    my $header = HTML::TableContent::Table::Header->new($_[1]);
+    push @{ $_[0]->headers }, $header;
+    return $header;
+}
+
 sub header_count { return scalar @{ $_[0]->headers }; }
 
 sub get_header { return $_[0]->headers->[ $_[1] ]; }
 
 sub get_first_header { return $_[0]->get_header(0); }
 
-around has_nested => sub {
-    my ( $orig, $self ) = ( shift, shift );
+sub get_last_header { return $_[0]->get_header($_[0]->header_count - 1); }
 
-    my $nested = $self->$orig(@_);
+sub clear_header { return splice @{ $_[0]->headers }, $_[1], 1; }
 
-    my $row = $self->get_first_row;
+sub clear_first_header { return shift @{ $_[0]->headers } }
 
-    for ( $row->all_cells ) {
-        if ( $_->has_nested ) {
-            $nested = 1;
-        }
-    }
-
-    return $nested;
-};
+sub clear_last_header { return $_[0]->clear_header( $_[0]->header_count - 1 ); }
 
 around raw => sub {
     my ( $orig, $self ) = ( shift, shift );
@@ -57,19 +81,40 @@ around raw => sub {
 
     if ( defined $self->caption ) { $table->{caption} = $self->caption->text }
 
-    $table->{headers} = [];
-    foreach my $header ( $self->all_headers ) {
-        push @{ $table->{headers} }, $header->raw;
-    }
+    my @headers =  map { $_->raw } $self->all_headers;
+    $table->{headers} = \@headers;
 
-    $table->{rows} = [];
-    foreach my $row ( $self->all_rows ) {
-        push @{ $table->{rows} }, $row->raw;
-    }
+    my @rows =  map { $_->raw} $self->all_rows;
+    $table->{rows} = \@rows;
 
     return $table;
 };
 
+sub _render_element {
+    my $args = $_[0]->attributes;
+    
+    my @rows = ( );
+    
+    if ( $_[0]->has_caption ) {
+        push @rows, $_[0]->caption->render;    
+    }
+
+    if ( $_[0]->header_count ) {
+        my @headers = map { $_->render } $_[0]->all_headers;
+        my $headers = sprintf '%s' x @headers, @headers;
+        my $header_row = sprintf '<tr>%s</tr>', $headers; 
+        push @rows, $header_row;
+    }
+    
+    if ($_[0]->row_count) {
+        push @rows, map { $_->render } $_[0]->all_rows;
+    }
+        
+    my $row = sprintf '%s' x @rows, @rows;
+
+    return $row;
+}
+
 sub headers_spec {
     my $self = shift;
 
@@ -120,6 +165,28 @@ sub get_header_column {
     return \@cells;
 }
 
+sub parse_to_column {
+    my ($self, $cell) = @_;
+    
+    my $row = $self->get_last_row;
+
+    my $header;
+    if ( $row->header ) {
+        $header = $row->header;
+    }
+    else {
+        my $cell_index = $row->cell_count;
+        $header = $self->headers->[$cell_index - 1];
+    }
+
+    return unless $header;
+
+    $cell->header($header);
+    push @{ $header->cells }, $cell;
+
+    return 1;
+}
+
 sub get_header_column_text {
     my ( $self, %args ) = @_;
     my $cells = $self->get_header_column(%args);
@@ -166,6 +233,7 @@ sub _dedupe_object_array_not_losing_order {
     return @new_items;
 }
 
+
 sub _filter_headers {
     my ( $self, @headers ) = @_;
 
@@ -184,7 +252,30 @@ sub _filter_headers {
         $row->_filter_headers($headers);
     }
 
-    return;
+    return 1;
+}
+
+
+sub clear_column {
+    my ($self, @headers) = @_;
+
+    my @remove_cell;
+    for my $index (0 .. $self->header_count - 1) {
+        for (@headers) {
+            if ( $self->headers->[$index]->lc_text =~ m/$_/ims ){
+                $self->clear_header($index);
+                push @remove_cell, $index;
+            }
+        }
+    }
+
+    foreach my $row ($self->all_rows ) {
+        for (@remove_cell) {
+            $row->clear_cell($_);
+         }
+    }
+
+    return 1;
 }
 
 __PACKAGE__->meta->make_immutable;
@@ -199,7 +290,7 @@ HTML::TableContent::Table - Base class for table's
 
 =head1 VERSION
 
-Version 0.07
+Version 0.08
 
 =cut
 
@@ -234,7 +325,13 @@ Base class for Table's.
 
 Return underlying data structure
 
-    $row->raw
+    $table->raw
+
+=head2 render
+
+Render the table as HTML.
+
+    $table->render;
 
 =head2 attributes
 
@@ -260,6 +357,18 @@ Table caption if found, see L<HTML::TableContent::Caption>.
 
     $table->caption;
 
+=head2 add_caption
+
+Add a caption to the table.
+
+    $table->add_caption({})
+
+=head2 has_caption
+
+Boolean check, returns true if table has a caption.
+
+    $table->has_caption;
+
 =head2 headers
 
 Array Ref of L<HTML::TableContent::Header>'s
@@ -272,17 +381,71 @@ Array of L<HTML::TableContent::Header>'s
 
     $table->all_headers;
 
+=head2 add_header
+
+Add a L<HTML::TableContent::Header> to the table.
+
+    $table->add_header({});
+
 =head2 header_count
 
 Number of headers found in table
 
     $table->header_count;
 
+=head2 get_header
+
+Get header from table by index.
+
+    $table->get_header($index);
+
+=head2 get_first_header
+
+Get first header in the table.
+
+    $table->get_first_header;
+
+=head2 get_last_header
+
+Get last header in the table.
+
+    $table->get_last_header;
+
+=head2 clear_header
+
+Clear header by array index.
+
+    $table->clear_header($index);
+
+=head2 clear_first_header
+
+Clear first header in table.
+
+    $table->clear_first_header;
+
+=head2 clear_last_header
+
+Clear last header in table.
+
+    $table->clear_last_header;
+
+=head2 headers_spec 
+
+Hash containing headers and their occurence count..
+
+    $table->headers_spec;
+
+=head2 header_exists 
+
+Boolean check to see if passed in headers exist.
+
+    $table->header_exists(qw/Header/);
+
 =head2 get_header_column
 
-Returns an array that contains HTML::TableContent::Table::Row::Cell's which belong to that column.
+Returns an array that contains L<HTML::TableContent::Table::Row::Cell>'s which belong to that column.
 
-    $table->get_header_column(header => 'Savings');
+    $table->get_header_column(header => $string);
 
 Sometimes you may want to dedupe the column that is returned. This is done based on the cell's text value.
 
@@ -306,24 +469,18 @@ Shorthand for get_header_column_text(header => '')
 
     $table->get_col_text('Email');
 
-=head2 get_header
-
-Get header from table by index.
-
-    $table->get_header($index);
-
-=head2 get_first_header
-
-Get first header in the table.
-
-    $table->get_first_header;
-
 =head2 rows
 
 Array Ref of L<HTML::TableContent::Row>'s
 
     $table->rows;
 
+=head2 add_row
+
+Add a L<HTML::TableContent::Row> to the table.
+
+    $table->add_row({});
+
 =head2 all_rows
 
 Array of L<HTML::TableContent::Row>'s
@@ -348,6 +505,85 @@ Get first row in the table.
 
     $table->get_first_row;
 
+=head2 get_last_row
+
+Get last row in the table.
+
+    $table->get_last_row;
+
+=head2 clear_row
+
+Clear row by index.
+
+    $table->clear_row($index);
+
+=head2 clear_first_row
+
+Clear first row in the table.
+
+    $table->clear_first_row;
+
+=head2 clear_last_row
+
+Clear last row in the table.
+
+    $table->clear_last_row;
+
+=head2 nested
+
+ArrayRef of all nested Tables found within the current table.
+
+    $table->nested
+
+=head2 all_nested
+
+Array of all nested Tables found within the current table.
+
+    $table->all_nested
+
+=head2 has_nested
+
+Boolean check, returns true if the table has nested tables.
+
+    $table->has_nested
+
+=head2 count_nested
+
+Count number of nested tables within current table.
+
+    $table->count_nested
+
+=head2 get_first_nested
+
+Get the first nested table.
+
+    $table->get_first_nested
+
+=head2 get_nested
+
+Get Nested table by index.
+
+    $table->get_nested(1);
+
+=head2 has_nested_table_column 
+
+Boolean Check, returns 1 if a headers cells consists of nested tables.
+
+    $table->has_nested_table_column;
+
+=head2 nested_column_headers
+
+Returns a hash,the header that contains nested tables and the a count of occurence.
+
+    $table->nested_column_headers;
+
+=head2 parse_to_column
+
+Magical. just call it if you want the header columns to work. 
+    
+    my $cell = $row->add_cell({ text => 'hacked' });
+    $table->parse_to_column($cell);
+    
 =head1 AUTHOR
 
 LNATION, C<< <thisusedtobeanemail at gmail.com> >>
