@@ -1,19 +1,28 @@
 package HTML::TableContent::Table::Header;
 
 use Moo;
+use HTML::TableContent::Table::Row::Cell;
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 
 extends 'HTML::TableContent::Element';
 
-__PACKAGE__->meta->make_immutable;
-
 has 'cells' => (
     is      => 'rw',
     lazy    => 1,
     default => sub { [] },
 );
 
+has '+html_tag' => (
+    default => 'th',
+);
+
+sub add_cell { 
+    my $cell = HTML::TableContent::Table::Row::Cell->new($_[1]);
+    push @{ $_[0]->cells }, $cell;
+    return $cell;
+}
+
 sub cell_count { return scalar @{ $_[0]->cells }; }
 
 sub all_cells { return @{ $_[0]->cells }; }
@@ -24,6 +33,8 @@ sub get_first_cell { return $_[0]->get_cell(0); }
 
 1;
 
+__PACKAGE__->meta->make_immutable;
+
 __END__
 
 =head1 NAME
@@ -32,7 +43,7 @@ HTML::TableContent::Table::Header - base class for table headers.
 
 =head1 VERSION
 
-Version 0.07 
+Version 0.08 
 
 =head1 SYNOPSIS
 
@@ -85,10 +96,16 @@ Get cell by Array index.
 
 =head2 get_first_cell
 
-Get first cell from cells
+Get first cell from cells.
 
     $header->get_first_cell;
 
+=head2 get_last_cell
+
+Get last cell from cells.
+
+    $header->get_last_cell;
+
 =head2 data
 
 ArrayRef of Text elements
