@@ -2,7 +2,10 @@ package HTML::TableContent::Table::Row;
 
 use Moo;
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
+
+use HTML::TableContent::Table::Header;
+use HTML::TableContent::Table::Row::Cell;
 
 extends 'HTML::TableContent::Element';
 
@@ -17,6 +20,22 @@ has header => (
     lazy => 1,
 );
 
+has '+html_tag' => (
+    default => 'tr',
+);
+
+sub add_header { 
+    my $header = HTML::TableContent::Table::Header->new($_[1]);
+    $_[0]->header($header); 
+    return $header;
+}
+
+sub add_cell { 
+    my $cell = HTML::TableContent::Table::Row::Cell->new($_[1]);
+    push @{ $_[0]->cells }, $cell;
+    return $cell;
+}
+
 sub cell_count { return scalar @{ $_[0]->cells }; }
 
 sub all_cells { return @{ $_[0]->cells }; }
@@ -27,9 +46,14 @@ sub get_first_cell { return $_[0]->get_cell(0); }
 
 sub get_last_cell { return $_[0]->get_cell( $_[0]->cell_count - 1 ); }
 
+sub clear_cell { return splice @{ $_[0]->cells }, $_[1], 1; }
+
+sub clear_first_cell { return shift @{ $_[0]->cells }; }
+
+sub clear_last_cell { return $_[0]->clear_cell( $_[0]->cell_count - 1 ); }
+
 sub _filter_headers {
     my ( $self, $headers ) = @_;
-
     my $cells = [];
     foreach my $cell ( $self->all_cells ) {
         for ( @{$headers} ) {
@@ -45,10 +69,10 @@ around raw => sub {
     my ( $orig, $self ) = ( shift, shift );
 
     my $row = $self->$orig(@_);
-    $row->{cells} = [];
-    foreach my $cell ( $self->all_cells ) {
-        push @{ $row->{cells} }, $cell->raw;
-    }
+
+    my @cells = map { $_->raw } $self->all_cells;
+    $row->{cells} = \@cells;
+    
     return $row;
 };
 
@@ -66,6 +90,12 @@ around has_nested => sub {
     return $nested;
 };
 
+sub _render_element {
+    my @cells = map { $_->render } $_[0]->all_cells;
+    my $cell = sprintf '%s' x @cells, @cells;
+    return $cell;
+};
+
 __PACKAGE__->meta->make_immutable;
 
 1;
@@ -78,7 +108,7 @@ HTML::TableContent::Table::Row - base class for table rows.
 
 =head1 VERSION
 
-Version 0.07
+Version 0.08
 
 =head1 SYNOPSIS
 
@@ -104,41 +134,17 @@ base class for rows
 
 =head1 SUBROUTINES/METHODS
 
-=head2 cells
-
-ArrayRef of L<HTML::TableContent::Row::Cell>'s
-
-    $row->cells;
-
-=head2 all_cells
-
-Array of L<HTML::TableContent::Row::Cell>'s
-
-    $row->all_cells;
-
-=head2 get_cell
-
-Get Cell from Row by index.
-
-    $row->get_cell($index);
-
-=head2 get_first_cell
-
-Get the first Cell from the Row.
-
-    $row->get_first_cell;
-
-=head2 cell_count
-
-Count number of Cell's inside the Row.
-
-    $row->cell_count
-
 =head2 raw
 
 Return underlying data structure
 
-    $row->raw
+    $row->raw;
+
+=head2 render
+
+Render the row as html.
+
+    $row->render;
 
 =head2 attributes
 
@@ -158,6 +164,72 @@ Row tag id if found.
 
     $row->id;
 
+=head2 cells
+
+ArrayRef of L<HTML::TableContent::Row::Cell>'s
+
+    $row->cells;
+
+=head2 all_cells
+
+Array of L<HTML::TableContent::Row::Cell>'s
+
+    $row->all_cells;
+
+=head2 add_cell
+
+Add a L<HTML::TableContent::Row::Cell> to the row.
+
+    $row->add_cell({ id => 'cell-1', text => 'some text' });
+
+=head2 cell_count
+
+Count number of Cell's inside the Row.
+
+    $row->cell_count
+
+=head2 get_cell
+
+Get Cell from Row by index.
+
+    $row->get_cell($index);
+
+=head2 get_first_cell
+
+Get the first Cell from the Row.
+
+    $row->get_first_cell;
+
+=head2 get_last_cell
+
+Get the last Cell from the Row.
+
+    $row->get_last_cell;
+
+=head2 clear_cell
+
+Clear cell from row by index.
+
+    $row->clear_cell($index);
+
+=head2 clear_first_cell
+
+Clear the first cell from the Row.
+
+    $row->clear_first_cell;
+
+=head2 clear_last_cell
+
+Clear the last cell from the Row.
+
+    $row->clear_last_cell;
+
+=head2 has_nested
+
+Boolean returns true if the row cell's has nested tables.
+
+    $row->has_nested;
+
 =head1 AUTHOR
 
 LNATION, C<< <thisusedtobeanemail at gmail.com> >>
