@@ -2,11 +2,29 @@ package HTML::TableContent::Table::Row::Cell;
 
 use Moo;
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 
 extends 'HTML::TableContent::Element';
 
-has header => ( is => 'rw', );
+has header => ( is => 'rw' );
+
+has '+html_tag' => (
+    default => 'td',
+);
+
+around _render_element => sub {
+    my ( $orig, $self ) = (shift, shift); 
+
+    my $text = $self->$orig(@_);
+
+    if ($self->has_nested) {
+        my @nested = map { $_->render } $self->all_nested;
+        my $nest = sprintf '%s' x @nested, @nested;
+        $text = sprintf '%s%s', $text, $nest;
+    } 
+        
+    return $text;
+};
 
 __PACKAGE__->meta->make_immutable;
 
@@ -20,7 +38,7 @@ HTML::TableContent::Table::Row::Cell - base class for table cells.
 
 =head1 VERSION
 
-Version 0.07
+Version 0.08
 
 =head1 SYNOPSIS
 
@@ -90,6 +108,48 @@ Cell tag id if found.
 
     $cell->id;
 
+=head2 nested
+
+ArrayRef of nested Tables.
+
+    $cell->nested
+
+=head2 all_nested
+
+Array of nested Tables.
+
+    $cell->all_nested
+
+=head2 has_nested
+
+Boolean check, returns true if the cell has nested tables.
+
+    $cell->has_nested
+
+=head2 count_nested
+
+Count number of nested tables.
+
+    $cell->count_nested
+
+=head2 get_first_nested
+
+Get the first nested table.
+
+    $cell->get_first_nested
+
+=head2 get_nested
+
+Get Nested table by index.
+
+    $cell->get_nested(1);
+
+=head2 add_nested
+
+Add nested L<HTML::TableContent::Table> to the Cell.
+
+    $cell->add_nested({ id => 'nested-table-id', class => 'nested-table-class' }); 
+
 =head1 AUTHOR
 
 LNATION, C<< <thisusedtobeanemail at gmail.com> >>
